@@ -43,7 +43,6 @@ from photutils.background import Background2D, MedianBackground
 from astropy.io import fits
 from astropy.convolution import Gaussian2DKernel, convolve
 from astropy.stats import sigma_clipped_stats, gaussian_fwhm_to_sigma, sigma_clip, SigmaClip
-from astroquery.astrometry_net import AstrometryNet
  
 #######################################################################
 # Initial set of parameters.
@@ -138,6 +137,11 @@ AstrometryNet_API_key = 'ujmrvwqqyelxmzcj'
 #######################################################################
 
 
+def weighted_centre(x_coords, y_coords, weights):
+    weighted_x = np.sum(np.array(x_coords) * np.array(weights)) / np.sum(weights)
+    weighted_y = np.sum(np.array(y_coords) * np.array(weights)) / np.sum(weights)
+    return (weighted_x, weighted_y)
+
 def read_columns(events_list):
     # Reading few columns.
     f = fits.open(events_list)
@@ -226,7 +230,8 @@ def auto_bg(fx, fy, time, photons, radius, framecount_per_sec, sky_radius):
     y_mesh = y_mesh.flatten()
     # To avoid the edges. 
     mask_radius = 1700
-    mask = (x_mesh - 2400) ** 2 + (y_mesh - 2400) ** 2 <= mask_radius ** 2
+    image_x_centre, image_y_centre = weighted_centre(fx, fy, weights)
+    mask = (x_mesh - image_x_centre) ** 2 + (y_mesh - image_y_centre) ** 2 <= mask_radius ** 2
     array = np.array([flat_counts, x_mesh, y_mesh]).T 
     polished_array = array[mask]
     bg_mask = sigma_clip(polished_array[:, 0], sigma = 3, maxiters = 5)
@@ -298,10 +303,13 @@ def create_sub_image(pos_x, pos_y,
 
 # To find positions of interest (using daofind algorithm).
 def detect_sources_daofind(fx, fy, photons, threshold):
+    weights = photons / framecount_per_sec
+    image_x_centre, image_y_centre = weighted_centre(fx, fy, weights)
+    
     mask_radius = 1700
     kernel = Gaussian2DKernel(x_stddev = 4 * gaussian_fwhm_to_sigma)
 
-    aperture = CircularAperture((2400, 2400), r = mask_radius)
+    aperture = CircularAperture((image_x_centre, image_y_centre), r = mask_radius)
     mask = aperture.to_mask(method = 'center')
     mask = mask.to_image(shape = ((4800, 4800)))
 
@@ -1238,8 +1246,10 @@ def combine_events_lists(events_lists_paths = None,
                          threshold = 8,
                          NUM_NEAREST_NEIGHBORS = NUM_NEAREST_NEIGHBORS,
                          MIN_MATCHES_FRACTION = MIN_MATCHES_FRACTION,
+                         minimum_detections = 3,
+                         maximum_detections = 100,
                          shift_algorithm = shift_algorithm, 
-                         min_exptime = min_exptime,
+                         min_exptime = min_exptime,                         
                          framecount_per_sec = framecount_per_sec):
                          
     """Align and combine the events lists from different orbits.
@@ -1251,9 +1261,7 @@ def combine_events_lists(events_lists_paths = None,
         
     threshold : float, optional
         The threshold parameter associated with the source detection method. 
-        The default value is 8. The threshold is lowered in steps of 0.5
-        until a minimum number of sources are detected. The minimum number
-        depends on the **shift_algorithm**.  
+        The default value is 8. 
         
     NUM_NEAREST_NEIGHBORS : int, optional    
         See https://astroalign.quatrope.org/en/latest/api.html#astroalign.NUM_NEAREST_NEIGHBORS
@@ -1262,6 +1270,14 @@ def combine_events_lists(events_lists_paths = None,
     MIN_MATCHES_FRACTION : float, optional    
         See https://astroalign.quatrope.org/en/latest/api.html#astroalign.MIN_MATCHES_FRACTION
         for details. The default value is 0.01. Only relevant for ``'multiple_star'`` method.
+        
+    minimum_detections : int, optional
+        The minimum number of sources to be detected. The threshold will be 
+        modified to meet the criteria. 
+
+    maximum_detections : int, optional
+        The maximum number of sources to be detected. The threshold will be 
+        modified to meet the criteria.         
                         
     shift_algorithm : {'single_star', 'multiple_star'}, optional
         The parameter to choose between available aligning methods. 
@@ -1271,10 +1287,7 @@ def combine_events_lists(events_lists_paths = None,
             in the field and no rotation between frames. The minimum number of
             source detection is 2. 
         * ``'multiple_star'``: To use multiple stars for aligning orbits. 
-            This is the **recommended** and default method. The minimum 
-            number of source detection is 3. The function will automatically 
-            modify the threshold to limit the maximum number of detections 
-            to below 200. 
+            This is the **recommended** and default method. 
                                
     min_exptime : float, optional
         Orbits having exposure time below this limit will be ignored. 
@@ -1314,39 +1327,44 @@ def combine_events_lists(events_lists_paths = None,
 
     exptimes = []
     detected_sources = []
-    select_events_lists_paths = []    
+    select_events_lists_paths = []   
+    default_threshold = threshold  
     for path in events_lists_paths:
+        threshold = default_threshold
         time, fx, fy, photons = read_columns(path)
         nbin_check = (time.max() - time.min()) / min_exptime
         if nbin_check < 1:
             print('\n{} ignored.\nExposure time below {} seconds.'.format(path, min_exptime))
             continue
-            
+       
+        # TODO: The following seems like a bad way to do this. Fix it later    
         uA = []    
         if shift_algorithm == 'single_star':
-            number_of_sources = 2
-            i = threshold
-            while len(uA) <= number_of_sources:
-                uA = new_detect_sources_daofind(fx, fy, photons, i, framecount_per_sec)
-                i = i - 0.5
-                if i <= 1:
+            while len(uA) <= minimum_detections:
+                uA = new_detect_sources_daofind(fx, fy, photons, threshold, framecount_per_sec)
+                threshold = threshold - 0.5
+                if threshold <= 1:
                     print('If you see this, please contact Curvit developer.')
-                    break
+                    break     
+            while len(uA) > maximum_detections:
+                uA = new_detect_sources_daofind(fx, fy, photons, threshold, framecount_per_sec)
+                threshold = 2 * threshold
+                if threshold >= 1e20:
+                    print('If you see this, please contact Curvit developer.')
+                    break                                   
         else:
-            number_of_sources = 3
-            i = threshold
-            while len(uA) <= number_of_sources:
-                uA = new_detect_sources_daofind(fx, fy, photons, i, framecount_per_sec)
-                i = i - 0.5
-                if i <= 1:
+            while len(uA) <= minimum_detections:
+                uA = new_detect_sources_daofind(fx, fy, photons, threshold, framecount_per_sec)
+                threshold = threshold - 0.5
+                if threshold <= 1:
                     print('If you see this, please contact Curvit developer.')
                     break  
-            while len(uA) > 200:
-                uA = new_detect_sources_daofind(fx, fy, photons, i, framecount_per_sec)
-                i = i + i
-                if i >= 1e20:
+            while len(uA) > maximum_detections:
+                uA = new_detect_sources_daofind(fx, fy, photons, threshold, framecount_per_sec)
+                threshold = 2 * threshold
+                if threshold >= 1e20:
                     print('If you see this, please contact Curvit developer.')
-                    break
+                    break                   
 
         print('{} sources detected in {}'.format(len(uA), path))
 
@@ -1369,11 +1387,16 @@ def combine_events_lists(events_lists_paths = None,
                 transf, (source_list, target_list) = aa.find_transform(elist, reference, max_control_points = 100)  
 
                 img_path = ntpath.split(path)[0] + '/*I_l2img*'
+                eventslist_header = fits.getheader(path)
                 if len(glob(img_path)) == 1:
                     img_hdu = fits.open(glob(img_path)[0])
                     framerate_from_header.append(1 / img_hdu[0].header['INT_TIME'])
                     RA_pointing = img_hdu[0].header['RA_PNT']
                     DEC_pointing = img_hdu[0].header['DEC_PNT']
+                elif 'AVGFRMRT' in eventslist_header:
+                    framerate_from_header.append(eventslist_header['AVGFRMRT'])
+                    RA_pointing = eventslist_header['RA_PNT']
+                    DEC_pointing = eventslist_header['DEC_PNT']    
                 else:
                     print('Requires exactly one file matching *I_l2img* in directory {}!'.format(ntpath.split(path)[0]))
                     sys.exit()
@@ -1446,11 +1469,16 @@ def combine_events_lists(events_lists_paths = None,
         for shift, path in zip(shifts, select_events_lists_paths):
 
             img_path = ntpath.split(path)[0] + '/*I_l2img*'
+            eventslist_header = fits.getheader(path)
             if len(glob(img_path)) == 1:
                 img_hdu = fits.open(glob(img_path)[0])
                 framerate_from_header.append(1 / img_hdu[0].header['INT_TIME'])
                 RA_pointing = img_hdu[0].header['RA_PNT']
                 DEC_pointing = img_hdu[0].header['DEC_PNT']
+            elif 'AVGFRMRT' in eventslist_header:
+                framerate_from_header.append(eventslist_header['AVGFRMRT'])
+                RA_pointing = eventslist_header['RA_PNT']
+                DEC_pointing = eventslist_header['DEC_PNT']                    
             else:
                 print('Requires exactly one file matching *I_l2img* in directory {}!'.format(ntpath.split(path)[0]))
                 sys.exit()
@@ -1557,6 +1585,8 @@ def image_astrometry(UV_image = None, threshold = 3, API_key = AstrometryNet_API
     >>> import curvit
     >>> curvit.image_astrometry('test.fits')        
     """    
+    from astroquery.astrometry_net import AstrometryNet
+    
     hdu = fits.open(UV_image, mode = 'update')
     sources = daofind_on_image_data(hdu[0].data, threshold)
     sources = sources + 1
